//=============================================================================
// Copyright (c) 2016-present Allan CORNET (Nelson)
//=============================================================================
// This file is part of the Nelson.
//=============================================================================
// LICENCE_BLOCK_BEGIN
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// Alternatively, you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program. If not, see <http://www.gnu.org/licenses/>.
// LICENCE_BLOCK_END
//=============================================================================
#define _CRT_SECURE_NO_WARNINGS
//=============================================================================
#include "IsEmptyScriptFile.hpp"
#include "characters_encoding.hpp"
//=============================================================================
namespace Nelson {
bool
IsEmptyScriptFile(const std::wstring& filename)
{
    FILE* fr;
#ifdef _MSC_VER
    fr = _wfopen(filename.c_str(), L"rt");
#else
    fr = fopen(wstring_to_utf8(filename).c_str(), "rt");
#endif
    if (fr != nullptr) {
        int ch;
        while (ch = getc(fr)) {
            if (ch == EOF) {
                break;
            }
            bool isCharManaged = (ch == ' ') || (ch == '\r') || (ch == '\n');
            if (!isCharManaged) {
                fclose(fr);
                return false;
            }
        }
        fclose(fr);
    }
    return true;
}
//=============================================================================
} // namespace Nelson
//=============================================================================
